@@ -1,3 +1,5 @@
+from typing import Optional
+
 from pyspark.sql import *
 
 
@@ -98,3 +100,52 @@ def collectMetrics(spark: SparkSession, df: DataFrame, subgraph: str, component:
 def createEventSendingListener(spark: SparkSession, execution_url: str, session: str, scheduled: bool):
     spark.sparkContext._jvm.org.apache.spark.sql.MetricsCollector.addSparkListener(
         spark._jsparkSession, execution_url, session, scheduled)
+
+
+def postDataToSplunk(props: dict, payload):
+    import gzip
+    import requests
+    from requests import HTTPError
+    from requests.adapters import HTTPAdapter
+    from urllib3 import Retry
+
+    session = requests.Session()
+    adapter = HTTPAdapter(max_retries=Retry(total=int(props.get("maxRetries", 4)),
+                                            backoff_factor=float(props.get("backoffFactor", 1)),
+                                            status_forcelist=[429, 500, 502, 503, 504]))
+    session.mount("http://", adapter)
+    session.headers.update({"Authorization": "Splunk " + props["token"],
+                            "Content-Encoding": "gzip",
+                            "BatchId": props.get("batchId", None)})
+
+    res = session.post(props["url"], gzip.compress(bytes(payload, encoding='utf8')))
+    if res.status_code != 200 and props.get("stopOnFailure", False):
+        raise HTTPError(res.reason)
+
+
+def splunkHECForEachWriter(props: dict):
+    def wrapper(batchDF: DataFrame, batchId: int):
+        max_load: Optional[int] = props.get("maxPayload")
+        # Take 90% of the payload limit and convert KB into Bytes
+        max_load = int(0.9 * 1024 * int(max_load)) if max_load else None
+        props.update({"batchId": str(batchId)})
+
+        def f(iterableDF):
+            payload, prevsize = "", 0
+
+            for row in iterableDF:
+                if max_load and prevsize + len(row) >= max_load:
+                    print(f"buffer hit at size {prevsize}")
+                    postDataToSplunk(props, payload)
+                    payload, prevsize = "", 0
+                else:
+                    payload += '{"event":' + row + '}'
+                    prevsize += len(row) + 10   # 10 bytes is for padding
+
+            if payload:
+                print(f"last payload with size {prevsize}")
+                postDataToSplunk(props, payload)
+
+        batchDF.toJSON().foreachPartition(f)
+
+    return wrapper
